package cn.com.qmth.examcloud.ws.starter.core;

import java.util.Map;

import javax.websocket.Session;

import org.apache.commons.io.IOUtils;

import com.google.common.collect.Maps;

/**
 * websocket session管理
 *
 * @author WANGWEI
 * @date 2019年11月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class SessionHolder {

	private static final Map<String, Session> INDEX_SESESION_MAP = Maps.newConcurrentMap();

	private static final Map<Session, String> SESESION_INDEX_MAP = Maps.newConcurrentMap();

	private static final Map<Session, String> SESSION_KEY_MAP = Maps.newConcurrentMap();

	private static final Map<Session, String> SESSION_TOKEN_MAP = Maps.newConcurrentMap();

	/**
	 * 重新设置session
	 *
	 * @author WANGWEI
	 * @param path
	 * @param key
	 * @param token
	 * @param session
	 */
	public static void setSession(String path, String key, String token, Session session) {
		String index = key + ":" + path;
		Session oldSession = INDEX_SESESION_MAP.get(index);
		if (null != oldSession && !oldSession.equals(session)) {
			SESESION_INDEX_MAP.remove(oldSession);
			SESSION_KEY_MAP.remove(oldSession);
			SESSION_TOKEN_MAP.remove(oldSession);
			IOUtils.closeQuietly(oldSession);
		}

		INDEX_SESESION_MAP.put(index, session);
		SESESION_INDEX_MAP.put(session, index);

		SESSION_KEY_MAP.put(session, key);
		SESSION_TOKEN_MAP.put(session, token);
	}

	/**
	 * 删除 session
	 *
	 * @author WANGWEI
	 * @param session
	 */
	public static void delSession(Session session) {
		String sessionId = SESESION_INDEX_MAP.get(session);
		if (null != sessionId) {
			INDEX_SESESION_MAP.remove(sessionId);
		}

		SESESION_INDEX_MAP.remove(session);
		SESSION_KEY_MAP.remove(session);
		SESSION_TOKEN_MAP.remove(session);

		IOUtils.closeQuietly(session);
	}

	/**
	 * 获取 session
	 *
	 * @author WANGWEI
	 * @param path
	 * @param key
	 * @return
	 */
	public Session getSession(String path, String key) {
		String index = key + ":" + path;
		Session session = INDEX_SESESION_MAP.get(index);
		return session;
	}

	/**
	 * 获取 key
	 *
	 * @author WANGWEI
	 * @param session
	 * @return
	 */
	public static String getKey(Session session) {
		return SESSION_KEY_MAP.get(session);
	}

	/**
	 * 获取 token
	 *
	 * @author WANGWEI
	 * @param session
	 * @return
	 */
	public static String getToken(Session session) {
		return SESSION_TOKEN_MAP.get(session);
	}
}
